@font-face {
  font-family: "Material Icons";
  src: url("./material-icons-LEZCGFVT.woff2") format("woff2"),
       url("./material-icons-JLIDJUWE.woff") format("woff");
  font-weight: normal;
  font-style: normal;
}

@font-face {
  font-family: "Material Icons Outlined";
  src: url("./material-icons-outlined-7BWLPMFK.woff2") format("woff2"),
       url("./material-icons-outlined-PCUTWIDZ.woff") format("woff");
  font-weight: normal;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-latin-400-normal-LOX3CHMS.woff2") format("woff2"),
       url("./roboto-latin-400-normal-TWCS3G4O.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-latin-ext-400-normal-JLTDD7L3.woff2") format("woff2"),
       url("./roboto-latin-ext-400-normal-DKWFTT22.woff") format("woff");
  font-weight: 400;
  font-style: normal;
  unicode-range: U+0100-024F;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-cyrillic-400-normal-V3H5IIDP.woff2") format("woff2"),
       url("./roboto-cyrillic-400-normal-JZANGCVN.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-cyrillic-ext-400-normal-J2JSVX6B.woff2") format("woff2"),
       url("./roboto-cyrillic-ext-400-normal-37DU6NPA.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-greek-400-normal-VPVGP5YU.woff2") format("woff2"),
       url("./roboto-greek-400-normal-S2O6A3MB.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-greek-ext-400-normal-IONFYYIZ.woff2") format("woff2"),
       url("./roboto-greek-ext-400-normal-AFHRTL5D.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-math-400-normal-M62DA447.woff2") format("woff2"),
       url("./roboto-math-400-normal-DRZ46ZLW.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-symbols-400-normal-RS3SF2FB.woff2") format("woff2"),
       url("./roboto-symbols-400-normal-ZCAYNMUT.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-vietnamese-400-normal-KACKQ7ZL.woff2") format("woff2"),
       url("./roboto-vietnamese-400-normal-R3IJFZXV.woff") format("woff");
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-latin-700-normal-T27NHG46.woff2") format("woff2"),
       url("./roboto-latin-700-normal-FTSWD4R3.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-latin-ext-700-normal-RZWHNB6B.woff2") format("woff2"),
       url("./roboto-latin-ext-700-normal-QRYVNM5E.woff") format("woff");
  font-weight: 700;
  font-style: normal;
  unicode-range: U+0100-024F;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-cyrillic-700-normal-SC4KOBVO.woff2") format("woff2"),
       url("./roboto-cyrillic-700-normal-KQC5BFSE.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-cyrillic-ext-700-normal-EFD6V7AP.woff2") format("woff2"),
       url("./roboto-cyrillic-ext-700-normal-W6MZMV6P.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-greek-700-normal-RO7FC64R.woff2") format("woff2"),
       url("./roboto-greek-700-normal-7TFLXFAP.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-greek-ext-700-normal-QK2I7OFR.woff2") format("woff2"),
       url("./roboto-greek-ext-700-normal-LYUZM53M.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-math-700-normal-7LZAYPTX.woff2") format("woff2"),
       url("./roboto-math-700-normal-JCFFZW2A.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-symbols-700-normal-2BHEMMHB.woff2") format("woff2"),
       url("./roboto-symbols-700-normal-H77MXCD5.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: "Roboto";
  src: url("./roboto-vietnamese-700-normal-LA5HL3OG.woff2") format("woff2"),
       url("./roboto-vietnamese-700-normal-7GAD6KHW.woff") format("woff");
  font-weight: 700;
  font-style: normal;
}
